unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls,
  RVStyle, RVScroll, RichView, RVEdit, RVNote, RVTable, CRVData, RVTypes,
  RVItem;


type
  TfrmMain = class(TForm)
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    Button1: TButton;
    rveNote: TRichViewEdit;
    rvsNote: TRVStyle;
    Button2: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure GenerateSampleDocument;
    function GenerateNote(const NoteId: String): Boolean;
    function MakeNoteStream(const NoteId: String): TStream;
    function MakeFootnote(const NoteId: String; TextStyleNo: Integer): TRVFootnoteItemInfo;
    function GetNoteTextStyleNo(StyleNo: Integer; RVStyle: TRVStyle): Integer;
    procedure ReplaceLinks(RVData: TCustomRVData);
  end;

var
  frmMain: TfrmMain;

implementation

uses PreviewFrm;

{$R *.dfm}

procedure TfrmMain.Button1Click(Sender: TObject);
begin
  ReplaceLinks(RichViewEdit1.RVData);
  RichViewEdit1.Format;
end;

procedure TfrmMain.Button2Click(Sender: TObject);
var
  frm: TfrmPreview;
begin
  frm := TfrmPreview.CreatePreview(RichViewEdit1);
  frm.ShowModal;
  frm.Free;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  GenerateSampleDocument;
end;

function TfrmMain.GenerateNote(const NoteId: String): Boolean;
begin
  Result := True;
  rveNote.Clear;
  if NoteId = 'link1' then
    with rveNote do
    begin
      AddNL('This note makes no sense, but it is still ', 0, 0);
      AddNL('necessary', 1, -1);
      AddNL('.', 0, -1);
    end
  else if NoteId = 'link2' then
    with rveNote do
    begin
      AddNL('This example should have at least ', 0, 0);
      AddNL('two', 1, -1);
      AddNL(' footnotes.', 0, -1);
    end
  else
    Result := False;
end;

procedure TfrmMain.GenerateSampleDocument;
begin
  with RichViewEdit1 do
  begin
    Clear;
    AddNL('In this document, there are some links', 0, 0);
    AddNLTag('(like this)', 1, -1, 'link1');
    AddNL('. We will change them to footnotes', 0, -1);
    AddNLTag('(and this link too)', 1, -1, 'link2');
    AddNL('.', 0, -1);
    Format;
  end;
end;

function TfrmMain.GetNoteTextStyleNo(StyleNo: Integer;
  RVStyle: TRVStyle): Integer;
var
  TextStyle: TFontInfo;
begin
  TextStyle := TFontInfo.Create(nil);
  TextStyle.Assign(RVStyle.TextStyles[StyleNo]);
  TextStyle.Color := clWindowText;
  TextStyle.Style := [];
  TextStyle.StyleEx := [];
  TextStyle.Jump := False;
  TextStyle.SubSuperScriptType := rvsssSuperScript;
  Result := RVStyle.FindTextStyle(TextStyle);
  TextStyle.Free;
end;

function TfrmMain.MakeNoteStream(const NoteId: String): TStream;
begin
  if GenerateNote(NoteId) then
  begin
    Result := TMemoryStream.Create;
    rveNote.SaveRVFToStream(Result, False);
    Result.Position := 0;
  end
  else
    Result := nil;
end;

function TfrmMain.MakeFootnote(const NoteId: String;
  TextStyleNo: Integer): TRVFootnoteItemInfo;
var
  Stream: TStream;
  NoteRef: TRVNoteReferenceItemInfo;
begin
  Result := nil;
  Stream := MakeNoteStream(NoteId);
  if Stream = nil then
    exit;
  Result := TRVFootnoteItemInfo.CreateEx(RichViewEdit1.RVData,
    GetNoteTextStyleNo(TextStyleNo, RichViewEdit1.Style), 1, False);

  NoteRef := TRVNoteReferenceItemInfo.CreateEx(Result.Document,
    GetNoteTextStyleNo(0, RichViewEdit1.Style));
  Result.Document.AddItem('', NoteRef);
  Result.Document.AddNL(' ', 0, -1);
  Result.Document.AppendRVFFromStream(Stream, -1);
  Result.Document.NormalizeDocument(0, True);

end;

procedure TfrmMain.ReplaceLinks(RVData: TCustomRVData);
var
  i, r, c: Integer;
  Table: TRVTableItemInfo;
  Note: TRVFootnoteItemInfo;
  s: TRVRawByteString;
begin
  for i := RVData.ItemCount - 1  downto 0 do
    if RVData.GetItem(i) is TRVTableItemInfo then
    begin
      Table := TRVTableItemInfo(RVData.GetItem(i));
      for r := 0 to Table.RowCount - 1 do
        for c := 0 to Table.ColCount - 1 do
          if Table.Cells[r, c] <> nil then
            ReplaceLinks(Table.Cells[r, c].GetRVData);
    end
    else if (RVData.GetItemStyle(i) >= 0) and
      RVData.GetRVStyle.TextStyles[RVData.GetItemStyle(i)].Jump then
    begin
      Note := MakeFootnote(RVData.GetItemTag(i), RVData.GetItemStyle(i));
      if Note <> nil then
      begin
        Note.ParaNo := RVData.GetItemPara(i);
        Note.ItemOptions := RVData.GetItemOptions(i) - [rvioUnicode];
        RVData.DeleteItems(i, 1);
        s := '';
        Note.Inserting(RVData, s, False);
        RVData.Items.InsertObject(i, s, Note);
        Note.Inserted(RVData, i);
        RVData.DoOnAddSpecialItem(i);
      end;
    end;
end;


end.
